
void CertificateViewer::init(const Certificate &_certif,
    User *u,  const bool displayCRL) {
    
    
    
  user = u;
  figCert->hide();
  
  certif = new Certificate(_certif);
  createGeneralPage();
  
  createDetailsPage();
  
  createCertificationPathPage();
 
  if(displayCRL && certif->isIssued(*certif) != X509_V_OK){
  
      createCRLPage();   
      
  }     
  else {      
      tab->removePage(tab->page(3));
  }
  
}

/** 
 * General Panel displaying basic information about the certificate  
 * 
 */
void CertificateViewer::createGeneralPage()
{
  // Display of the usage of the  certificate
  QString usage;
  
  if(certif->isCA()) {
    usage = " - ";
    usage += "CA";
    usage += "\n";
  }

  if(certif->isForSigningAndEncrypting()) {
    usage += " - ";
    usage += "Signing";
    usage += "\n";
    usage +=" - ";
    usage += "Encrypting";
  } else if(certif->isForSigning()) {
    usage =" - ";
    usage += "Signing";
  } else if(certif->isForEncrypting()) {
    usage =" - ";
    usage += "Encrypting";
  }

  usageText->setText(usage);

  //Subject infos
  std::vector<std::string> subject_info;
  dn dn_subject = certif->getSubjectName();

  subject_info = dn_subject.getValuesVect();
  QString sinfo ="";
   
  for(std::vector<std::string>::iterator it = subject_info.begin(); it != subject_info.end() ; it++){
    sinfo+=" - ";
    sinfo+=it->c_str();
    sinfo+="\n";
  }
  
  sinfoText->setText(sinfo);
  
  //Issuer infos
  std::vector<std::string> issuer_info;
  dn dn_issuer = certif->getIssuerName();
  issuer_info = dn_issuer.getValuesVect();
  QString iinfo ="";
   
  for(std::vector<std::string>::iterator it = issuer_info.begin();
      it != issuer_info.end() ; it++){
    iinfo+=" - ";
    iinfo+=it->c_str();
    iinfo+="\n";
  }

  iinfoText->setText(iinfo);
  
  // Display of the date of issue and expiration

  QString issue ="";
  //   issue += "\n";
  issue += "De";
  issue += " : ";
  issue += certif->getPrettyStartDate().c_str();
  issue += "\n";
  issue += "At";
  issue += " : ";
  issue += certif->getPrettyEndDate().c_str();
  // _("This certificate is valid");
 
  issueText->setText(issue);
  
}

void CertificateViewer::createDetailsPage()
{

  // Display of the serial number
  serialNumber->setText(QString("Nmero de srie : ")+QString(certif->getSerialNumber().c_str()));

  // Address mail
  mail->setText(QString("EMail : ")+QString(certif->getEmailAddress().c_str()));

  // Hash of the certificate
  hash->setText(QString("Hash : ")+QString(certif->getHash().c_str()));   

  //X509 v3 extensions
  std::vector<std::string> ext = certif->getV3ext();
   
  QString extensions="";
  for(std::vector<std::string>::iterator it = ext.begin() ; it != ext.end() ; it++){
    extensions +=" - ";
    extensions +=it->c_str();
    extensions +="\n";
  }
   
  extensionsText->setText(extensions);
}

void CertificateViewer::createCertificationPathPage()
{
  std::vector<Certificate> *certList = new std::vector<Certificate>;
  Certificate *cert = NULL;

  
  
  if(user != NULL) {
     QString caDirName(user->getCACertificatesDir().c_str());
  
    
     QDir dirCAs(caDirName);
     QStringList caFilenames = dirCAs.entryList();
     QString fileName;
     if(certif != NULL) {
         for (QStringList::Iterator it = caFilenames.begin(); (it != caFilenames.end()); ++it ) {
            fileName = caDirName+QDir::separator()+*it;
            cert = new Certificate();
            if(cert->load(fileName.latin1())==SUCCESS) { 
                certList->push_back(*cert);
            }
          }
     }
  }    
  vect = certif->getPathToRoot(certList);
   
   bool firstTime = true;
   
   std::vector<Certificate>::iterator certIter;
   QListViewItem *certItem;
   certListComp->header()->hide();   
   unsigned int index = 0;
   //Cria as colunas dinamicas da Tree
   for (;index < vect->size(); index++) {
        QString nivel = QString("Nivel %1").arg(index) ;
       certListComp->addColumn(nivel);
 certListComp->header()->setLabel( index, nivel);
   }
   certListComp->setSorting(-1);
   
   for(certIter = vect->begin(); certIter!= vect->end();certIter++){
      /* Subject */
 std::string dnValue = certIter->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE, ' ');    
 
 if(firstTime) {
  certItem = new QListViewItem(certListComp, QString(dnValue.c_str()));
  firstTime = false;
 }
 else {
         certItem =  new QListViewItem(certItem, QString(dnValue.c_str()));
 } 
        certItem->setPixmap(0, image0);  
 certItem->setOpen(true); 
 
   }
 }

void CertificateViewer::createCRLPage(){  
  fillCrlList();  
}

void CertificateViewer::fillCrlList(){
    crlDate->setText("");
    crlVersion->setText("Verso : ");
    crlLastUpdate->setText("ltima atualizao :");
    crlNextUpdate->setText("Prxima atualizao :");
    crlUri->setText("");
    
    bool hasCRL = false;
    
    if( user->authorities->getAuthority
 ( certif->getIssuerName().getValues
   (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',')) != NULL )  {
   
 crlVector = user->authorities->getAuthority
     ( certif->getIssuerName().getValues
       (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','))->getAttributeValues( "CRL" );
 
 std::vector<std::string>::iterator vit;
 
 for(vit = crlVector.begin(); vit != crlVector.end() ; vit++) {
 
 QString crlFile = QString(user->getCRLDir().c_str())+QDir::separator()+QString(vit->c_str());
 
 
     Crl *c = new Crl();
     if(c->load(crlFile.latin1())==SUCCESS){
      crlAdd->setEnabled(false);
      uriUpdate->setEnabled(true);
      crlDelete->setEnabled(true);
 
      displayCrlInfo(QString(vit->c_str()));
 
      hasCRL = true;
     }
 }
    }
 
    if(!hasCRL){
 crlAdd->setEnabled(true);
 uriUpdate->setEnabled(true);
 crlDelete->setEnabled(false);
    } 
}



void CertificateViewer::displayCrlInfo(QString crlFile){
    
    if(!crlFile.isEmpty()) {      
       Crl *c = new Crl();
       QString crlFullPath = QString(user->getCRLDir().c_str())+QDir::separator()+crlFile;
       
       if ( c->load(crlFullPath.latin1()) == SUCCESS ){
        crlFileNameGb = crlFile;
 QString version;
 version.sprintf("%d",c->getVersion());
 crlVersion->setText(QString("Verso : "+version));
 crlDate->setText(QString(c->getLastUpdate().c_str()));
 crlLastUpdate->setText(QString("ltima atualizao : ")+QString(c->getLastUpdate().c_str()));
 crlNextUpdate->setText(QString("Prxima atualizao : ")+QString(c->getNextUpdate().c_str()));
        
 //finding uri index 
        if( user->authorities->getAuthority( certif->getIssuerName().getValues
          (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',')))
        {
  std::vector<std::string> crlValues = user->authorities->getAuthority
      ( certif->getIssuerName().getValues
        (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',') )->getAttributeValues( "CRL" );
  std::vector<std::string>::iterator valuesIt =  crlValues.begin();
  int i=0;
  while(valuesIt != crlValues.end() && *valuesIt != crlFile.latin1()){
      i++;
      valuesIt++;
  }
  std::vector<std::string> uriValues = user->authorities->getAuthority
      ( certif->getIssuerName().getValues
        (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',') )->getAttributeValues( "URI" );
  std::vector<std::string>::iterator uriIt  =  uriValues.begin();
  int j=0;
  while(i!=j){
      uriIt++;
      j++;
  }
  
  if( *uriIt == "none") {
      uriUpdate->setEnabled(false);
      crlUri->setEnabled(true);
      crlUri->setText("Esta CRL no est associada a uma url.");

  } else {
      crlUri->setText(QString(uriIt->c_str()));
      uriUpdate->setEnabled(true);
      crlUri->setEnabled(true);
  }
  
         }
       }
    }
    else {
        crlDate->setText("");
        crlVersion->setText("Verso : ");
        crlLastUpdate->setText("ltima atualizao :");
        crlNextUpdate->setText("Prxima atualizao :");
        crlUri->setText("");
    }
}



void CertificateViewer::crlDelete_clicked()
{
    
    AddCrlDlg::deleteCrl(crlFileNameGb , *certif , user );
    
    displayCrlInfo(QString(""));
    
    crlAdd->setEnabled(true);
    
    crlDelete->setEnabled(false);
    
}


bool CertificateViewer::saveCrl(Crl *crl, QString uri){
    int ret =  saveCrl( uri , crl , *certif , user );
    saveCrlErrorHandler( ret );
    return ret;
}


void CertificateViewer::saveCrlErrorHandler( int error ){

    switch( error ){
 
    case ERROR_CRL_WRONG_ISSUER: {
 QMessageBox::critical(this, "Assinador", "A CRL no foi emitida pela CA correspondente a este certificado."); 
 break;
    }
 
    case ERROR_CRL_SAVE_FILE:{
 QMessageBox::critical(this, "Assinador", "Ocorreu um erro ao tentar gravar CRL");
 break;
    }
 
    case  ERROR_CRL_CONFIG_UPDATE:{
 QMessageBox::critical(this, "Assinador", "Ocorreu um erro ao tentar gravar configurao de CRL");
 break;
    }

    case  ERROR_CRL_NOT_UNIQUE:{
 QMessageBox::information(this, "Assinador", "Esta CRL j est atualizada.");
 break;
    }

    case ERROR_CRL_NO_CA_ISSUER:{
 QMessageBox::critical(this, "Assinador", "No foi possvel encontrar o certificado da CA que emitiu a CRL");
 break;
    }

 
    }
    
}


QString CertificateViewer::fetchCrl(QString uri){
    return AddCrlDlg::fetchCrl( uri );
}

void CertificateViewer::checkCert()
{
    openViewerBtn->setEnabled(certListComp->childCount ()>0 && certListComp->selectedItem() != NULL);
}

void CertificateViewer::openViewerBtn_clicked()
{
   QListViewItem *certItem = certListComp->selectedItem();
   viewCert(certItem); 
}

void CertificateViewer::viewCert(QListViewItem *certItem)  
{
    if (certItem != NULL) {
      std::string dnValue = certif ->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE, ' ');    
       if(dnValue != certItem->text(0)) {
    if(certItem != NULL) {  
        std::vector<Certificate>::iterator certIter;  
    
        for(certIter = vect->begin(); certIter!= vect->end();certIter++){
      /* Subject */
     std::string dnValue = certIter->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE, ' ');    
      
     if(certItem->text(0) == dnValue) {      
         CertificateViewer cv(this);
         cv.init(*certIter, user, true);
         cv.exec();
     } 
        }
    }
       }
       }
}


void CertificateViewer::crlAdd_clicked()
{
   SelectCrlDlg crlDlg(this);
   if(crlDlg.exec() == QDialog::Accepted) {
      if(crlDlg.isFileMethod()) {
      	QString wildcard("Lista de Certificados Revogados (*.crl)");
     
     	QString crlFileName = QFileDialog::getOpenFileName(
                    QDir::homeDirPath(),
                    wildcard,
                     this,
                     QString("importa uma CRL"),
                     QString("Escolha um arquivo de CRL para importar"));

     	if( !crlFileName.isEmpty()) {
  	   Crl *crl = new Crl();
  	   QString uri="none";
  	   if(crl->load(crlFileName.latin1())==SUCCESS){ 
      	      saveCrl(crl, uri);
              fillCrlList();
  	   }    
     	}
      }
      else {
          std::string uri = certif->extractCrlUri();
          if( uri != ""){
               QString crlFile = AddCrlDlg::fetchCrl(uri.c_str() );
	   
     	       
               
               Crl newCrl;
	     
    	       if( newCrl.load( crlFile.latin1()) == SUCCESS ){
	   	
		  AddCrlDlg::saveCrl( uri.c_str() , &newCrl , *certif , user ); 
		
		  fillCrlList();
        
    	       } else {
        	  QMessageBox::critical(this, "Assinador",  "A CRL baixada  invlida.");
    	       }
	       
	       if (!crlFile.isEmpty()) {
	          //QFile removeme(newCrlFile);
	          //removeme.remove();
    	       }
	       
	     
          } else if(uri ==""){
             QMessageBox::information(this, "Assinador",  "Este certificado no possui CRL.\n Adicione a CRL manualmente.");       
          }
      
        
      }
   }
}

int CertificateViewer::saveCrl( QString uri , Crl *crl , Certificate c , User *user ){
   AddCrlDlg::saveCrl(uri, crl, c, user);
}

/*
int CertificateViewer::saveCrl( QString uri , Crl *crl , Certificate c , User *user ){

   // getting crl vector (filename)
    
    if( ! user->authorities->getAuthority(c.getIssuerName().getValues (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',')) ){
        //return ERROR_CRL_NO_CA_ISSUER;
    }
    std::vector<std::string> crlValues = user->authorities->getAuthority
	(c.getIssuerName().getValues
	 (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',') )->getAttributeValues( "CRL" );
    
    //build crl hash
    std::vector<std::string>::iterator valuesIt ;
    std::string hash = crl->getHash();
    hash +=".der";
    bool unique = true;
    
    //check if this hash is already present in the config file
    for(valuesIt = crlValues.begin() ; valuesIt != crlValues.end() ; valuesIt++){
	if(*valuesIt == hash) unique = false;
    }
    // loading available CAs
    std::vector<Certificate> *certList = new std::vector<Certificate>;
    std::vector<Certificate> *vect;
    
    QDir dirCAs(QString( user->getCACertificatesDir().c_str()));
    QStringList caFilenames = dirCAs.entryList();
    QString fileName;
    
    Certificate *cert = NULL;
    if(certif != NULL) {
      for ( QStringList::Iterator it = caFilenames.begin(); (it != caFilenames.end()); ++it ) {
	  QString fileName = *it;
	  cert = new Certificate();
	  if(cert->load(fileName.latin1())==SUCCESS) {
	      certList->push_back(*cert);
	  }
        }
    }
    vect = c.getPathToRoot(certList);
    std::vector<Certificate>::iterator certIt;
    
    //check if this crl has been issued for/by the current CA certificate.
    bool issuedOK = false;
    for(  certIt = vect->begin() ; certIt != vect->end() ; certIt++ ){
	Certificate *ce = new Certificate( *certIt );
	if(crl->isIssued( *ce )) { //&& c.isIssued( *ce )== X509_V_OK){
	    issuedOK = true ;
	    delete ce ;
	    break;
	}
	delete ce;
    }
    
    if(! issuedOK ){
	//return ERROR_CRL_WRONG_ISSUER;
    }
    
    
    //build crl filename
    QString crlFile = QString(user->getCRLDir().c_str())+QDir::separator()+QString(hash.c_str());
  
    if(unique) {

	// try to find if there is a crl with the same url
	// if we can find such crl, we replace it by the current 
	// new one 
	//
	std::vector<std::string> URIValues = user->authorities->getAuthority
	    (c.getIssuerName().getValues
	     (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',') )->getAttributeValues( "URI" );
	std::vector<std::string>::iterator uriIt = URIValues.begin();
	std::vector<std::string>::iterator hashIt = crlValues.begin();

	if( uri != "none" ){
	    int i=0;
	    if( URIValues.size() > 0) {
		while(uriIt != URIValues.end() && *uriIt != uri.latin1() ){
		    i++;
		    uriIt++;
		    hashIt++;
		}
		
		// this uri already exist, so we delete it and 
		// replace it with the new one
		//
		if(uriIt != URIValues.end()) {
		   if( *uriIt == uri.latin1()){
			URIValues.erase( uriIt  );
			crlValues.erase( hashIt );
		   }
		}
	    }
	}
	
	
    	if(crl->save(crlFile.latin1()) != SUCCESS){
	    return ERROR_CRL_SAVE_FILE;
	}
	
	//adding the new crl and its uri.
	crlValues.push_back( crlFile.latin1() );
	URIValues.push_back( uri.latin1() );
	

	//saving config
	if( ! user->authorities->setInfos
	    (c.getIssuerName().getValues
	     (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','), "CRL",
	     crlValues )){
	
	    return ERROR_CRL_CONFIG_UPDATE;
	}
	
	if( ! user->authorities->setInfos
	    (c.getIssuerName().getValues
	     (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','), "URI", URIValues )) {
	    return ERROR_CRL_CONFIG_UPDATE;
	}
	
	return true;    
    } else { // this crl is not unique (same hash found in config file

	return ERROR_CRL_NOT_UNIQUE;
    }
}
*/

void CertificateViewer::uriUpdate_clicked()
{
  if(!crlUri->text().isEmpty()){
 QString uri =  crlUri->text();
 
 
 int  ret =  AddCrlDlg::updateCrl( uri , *certif , user );
 saveCrlErrorHandler( ret  );
 
 switch( ret ){
 case ERROR_CRL_WRITE_FILE:{
     QMessageBox::critical(this, "Assinador", "Ocorreu um erro duranta a gravao da CRL em disco.");
     break;
 }

 case ERROR_CRL_LOAD:{
     QMessageBox::critical(this, "Assinador", "Ocorreu um erro durante o carregamento da CRL.");
     break;
 }
     
 }
 
 fillCrlList();  
    }
    
}
