/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qdir.h>
#include <qfile.h>
#include <qiconview.h>
// #include <qtextcodec.h>
#include <qlocale.h>

#include "AddIdentity.h"
#include "GenericDataListViewItem.h"
#include "CertificateRequestWizard.h"
#include "CertificateViewer.h"

#include "../Certificate.hh"
#include "../pkcs12.hh"
#include "../SoftKeyStore.hh"
#include "../tokenkeystore.h"
#include "../Utils.hh"
#include "AddCrlDlg.h"


/* Funcao utilitario: nao faz parte da classe
   Cuidado: o QT designer tenta colocar ela na lista de public functions
*/
/* Return the certificate's CommonName or the
 * organization name if there is no CN.
 */
 
    std::string getCertName( Cryptonit::Certificate cert )
{
  /* Use the origanization name, if there is no CN */
  std::string name = cert.getSubjectName().getValue("CN");
  if( name == "" )
  {
    name = cert.getSubjectName().getValue("O");
    if( cert.getSubjectName().getValue("OU") != "" )
      name += " / " + cert.getSubjectName().getValue("OU");
  }
  return name;
}
  
void PropertiesFrame::updateStack(QIconViewItem* icon)
{buildCATree();
  widgetStack1->raiseWidget(icon->index());
  helpText->setText(icon->text());
}


void PropertiesFrame::slotIdentities_add()
{
  AddIdentity addIdentity(this);
  addIdentity.init(user);

  if( addIdentity.exec() == QDialog::Accepted )
  {
    Cryptonit::Certificate c = addIdentity.getCertificate();
    Cryptonit::pkcs12 p12 = addIdentity.getKey();
    QString label = addIdentity.getIdentity();
    std::vector< Cryptonit::KeyStore *>::iterator i = keyStores->begin();
    if (c.getHash() != "") {
      if (label[0] != '_') {
         if (p12.getHash() != "") {
	        if ((*i)->getType() != SOFT_KEYSTORE) {
			i++;
		}
		(dynamic_cast<Cryptonit::SoftKeyStore*>(*i))->addPair( c , p12 );
		updateIDList(c , p12, label);
	 }
      }
      else {
      	label = label.remove(0, 1);
	if ((*i)->getType() != PKCS11_KEYSTORE) {
		i++;
	}
	TokenKeyStore *tokenStore = dynamic_cast<TokenKeyStore*>(*i);
	tokenStore->addPair( c , addIdentity.getKeyID() );
	updateIDList(c , p12, label);
      }
    }
  }
}


void PropertiesFrame::updateIDList(Cryptonit::Certificate &c, Cryptonit::pkcs12&, const QString &label)
{
  Cryptonit::Certificate cert = c;
  std::string* h = new std::string( c.getHash().c_str() );

  (void)new GenericDataListBoxItem(IDList, (void*)h, label);
}

void PropertiesFrame::slotIdentities_remove()
{
  std::vector< Cryptonit::KeyStore *>::iterator i = keyStores->begin();
  std::string test;
  int ret = (*i)->removeKey( IDList->currentItem() , test) ;
  if (ret != Cryptonit::SUCCESS && ret == -3) {
  	i++;
	ret = (*i)->removeKey( IDList->currentItem() , test) ;
  }

  switch( ret )
  {
  case -1:
    {
      QMessageBox::critical(this, "Erro", QString("No foi possvel apagar arquivo de certificado:")
                            + test.c_str());
    }
    break;
  case -2:
    {
      QMessageBox::critical(this, "Erro", QString("No foi possvel apagar arquivo PKCS#12:")
                            + test.c_str());
    }
    break;
  case -3:
    {
      //thiago: aff:

      /*wxMessageDialog errorMsg2(this, _("Out of bound"), _("Error"),
                                wxOK | wxICON_ERROR);
      errorMsg2.ShowModal();*/

    }
    break;

  case -7: std::cerr << "Canceled by user" << std::endl; break;

  default:
    {
      // refresh the listbox
      GenericDataListBoxItem* item =
        dynamic_cast<GenericDataListBoxItem*>(IDList->item(IDList->currentItem()));

      std::string *data = static_cast<std::string*>(item->getData());
      if( data )
        delete data;

      IDList->removeItem( IDList->currentItem() );
    }
    break;
  }
}


void PropertiesFrame::slotIdentities_detailes()
{
    if (IDList->currentItem() >= 0) {
	GenericDataListBoxItem* item = 
		dynamic_cast<GenericDataListBoxItem*>(IDList->item(IDList->currentItem()));
	
	std::string *h = static_cast<std::string*>(item->getData());
	if(h != NULL){
	QString certificateFilename ( h->c_str() );
	certificateFilename += ".der";
	certificateFilename.prepend( QDir::separator() );
	certificateFilename.prepend( user->getCertificatesDir().c_str() );
	
	Cryptonit::Certificate c;
		CertificateViewer cv(this);
	if( c.load( certificateFilename.latin1() ) == Cryptonit::SUCCESS){
		cv.init(c, user, true);
		cv.exec();
	}
    }
    }

}


void PropertiesFrame::slotIdentities_export()
{
  if(IDList->currentItem() == -1) return; //no item selected

  GenericDataListBoxItem* item =
    dynamic_cast<GenericDataListBoxItem*>(IDList->item(IDList->currentItem()));

  std::string *h = static_cast<std::string*>(item->getData());
  if( h )
  {
    QString wildcard = "PKCS#12 file (*.pkcs12 *.p12 *.pfx)";

    QString filename = QFileDialog::getSaveFileName(
                         QDir::homeDirPath(),
                         wildcard,
                         this,
                         "save file dialog",
                         "Exporte seus dados pessoais" );

    if ( !filename.isEmpty() )
    {
      QString pkcs12File( h->c_str() );
      pkcs12File += ".p12";
      pkcs12File.prepend("/");
      pkcs12File.prepend( user->getP12Dir().c_str() );

      QFile in(pkcs12File);
      QFile out(filename);

      if(!in.open(IO_ReadOnly) || !out.open(IO_WriteOnly))
      {
        QMessageBox::critical(this, "Erro", "Erro ao abrir arquivos.");
        return;
      }

      QByteArray data = in.readAll();
      bool success = static_cast<uint>(out.writeBlock(data)) == data.count();

      in.close();
      out.close();

      if(success)
      {
        QMessageBox::information(this, "Exportao completa",
                                 QString("Dados pessoais exportados para: ") +
                                 filename);
      }
      else
      {
        QMessageBox::critical(this, "Erro", "Ocorreu um erro durante a exportao.");
      }
    }
  }
}


void PropertiesFrame::slotIdentities_reqCert()
{
  CertificateRequestWizard* wizi = new CertificateRequestWizard(this);
  wizi->init(user);
  wizi->exec();
  delete wizi;
}


void PropertiesFrame::init( Cryptonit::User * u, std::vector< Cryptonit::KeyStore *> *_keyStores )
{
  keyStores = _keyStores;
  ks = getSoftKeyStore();
  user = u;
  //   caTree = NULL;
  //   caDNValue = NULL;
  //   caSNValue = NULL;
  //   caINValue = NULL;
  cipher = NULL;
  selectedCipher = "";
  selectedDigest = "";

  helpText->setText(iconView4->firstItem()->text());

  fillIDList();
  buildCATree();

  caTree->header()->hide();
  /*
  CryptonitGui& app = wxGetApp();
  langList = new wxListBox( panel, -1, wxDefaultPosition, wxDefaultSize,
  CryptonitGui::nbLangs, CryptonitGui::langs,
  wxLB_SINGLE | wxLB_NEEDED_SB | wxLB_SORT );  
  */
  lbLang->setText("Portugus do Brasil");
  langList->insertItem("Portugus do Brasil");

  btRemoveAuthority->setEnabled(false);
  //Algorithm
  int i = -1;

  cipher = new std::vector<std::string>;
  //   cipher_name = new QString[CIPHER_NUM];

  std::vector< std::pair< std::string, std::string> > allCiphers;
  std::vector< std::pair< std::string, std::string> >::iterator itr;
  allCiphers = getAllCiphers();

  for( itr = allCiphers.begin(); itr != allCiphers.end(); itr++ )
  {
    cipher_name.append(QString(itr->first.c_str()));
    ++i;
    cipher->push_back( itr->second );
  }


  int j=-1;
  digest = new std::vector<std::string>;
  //   digest_name = new QString[DIGEST_NUM];
  digest_name.clear();

  std::vector< std::pair< std::string, std::string> > allDigests;
  allDigests = getAllDigests();

  for( itr = allDigests.begin(); itr != allDigests.end(); itr++ )
  {
    digest_name.append(QString(itr->first.c_str()));
    ++j;
    digest->push_back( itr->second );
  }

  QString selectedCipherName;

  if(selectedCipher == "")
  {
    selectedCipherName = getCipherName(user->getInfo("CipherAlgo"));
    if(selectedCipherName =="")
    {
      selectedCipherName = "AES 128 bits";
    }
  }
  else
  {
    selectedCipherName = getCipherName(selectedCipher);
  }

  QString selectedDigestName;

  if(selectedDigest == "")
  {
    selectedDigestName= getDigestName(user->getInfo("DigestAlgo"));
    if(selectedDigestName == "")
    {
      selectedDigestName = "SHA1";
    }
  }
  else
  {
    selectedDigestName = getDigestName(selectedDigest);
  }

  if(i >= 0)
  {
    //     encryptAlgo->insertStringList(cipher_name);

    //setting up data
    std::vector<std::string>::iterator it;
    int k=0;

    for(it=cipher->begin(); it!=cipher->end();  it++)
    {
      std::string *data = new std::string(*it);
      (void)new GenericDataListBoxItem(encryptAlgo->listBox(), (void*)data, cipher_name[k]);
      k++;
    }
    encryptAlgo->setCurrentText(selectedCipherName);
  }
  else
  {
    QMessageBox::critical(this, "Erro", "Nenhum algoritmo de encriptao foi encontrado.");
  }

  //   signatureAlgo->insertStringList(digest_name);

  std::vector<std::string>::iterator it;
  j=0;

  for(it=digest->begin(); it!=digest->end();  it++)
  {
    std::string *data = new std::string(*it);
    (void)new GenericDataListBoxItem(signatureAlgo->listBox(), (void*)data, digest_name[j]);
    j++;
  }
  signatureAlgo->setCurrentText(selectedDigestName);
}

void PropertiesFrame::fillIDList()
{
  std::vector< Cryptonit::KeyStore *>::iterator i = keyStores->begin();
  std::vector<Cryptonit::Certificate> certsSoft = (*i)->listCertificates();
  i++;
  std::vector<Cryptonit::Certificate> certsToken = (*i)->listCertificates();
  std::vector<std::string> identities = user->getInfos( "Identities" );
  unsigned int posSoft = 0;
  unsigned int posToken = 0;
  Certificate cert;
  for(int identityIndex = 0; identityIndex < identities.size(); identityIndex++)  
  {
    std::string identityName = identities[identityIndex];
    if (identityName[0] == '_') {
    	cert = certsToken[posToken++];
	identityName = identityName.substr(1, identityName.size() - 1);
    }
    else {
    	cert = certsSoft[posSoft++];
    }
    std::string* h = new std::string( cert.getHash() );
    if( *h != "") {
      (void)new GenericDataListBoxItem(IDList, (void*)h, identityName.c_str());
    }
  }
}

QString PropertiesFrame::getCipherName( std::string c )
{
  int i=0;
  for(std::vector<std::string>::iterator it = cipher->begin(); it!=cipher->end(); it++)
  {
    if(c == *it)
      return QString(cipher_name[i]);
    i++;
  }
  return QString::null;
}


QString PropertiesFrame::getDigestName( std::string c )
{
  int i=0;
  for(std::vector<std::string>::iterator it = digest->begin(); it!=digest->end(); it++)
  {
    if(c == *it) return QString(digest_name[i]);
    i++;
  }
  return QString::null;
}


Cryptonit::SoftKeyStore* PropertiesFrame::getSoftKeyStore()
{
  std::vector< Cryptonit::KeyStore *>::iterator i;
  for( i = keyStores->begin() ; i != keyStores->end() ; i++ )
  {
    if ( (*i)->getType() == Cryptonit::SOFT_KEYSTORE )
      return (Cryptonit::SoftKeyStore*)*i;
  }
  return NULL;
}


void PropertiesFrame::slotAdd()
{
  applyLanguages();

  //aplicar outras coisas aqui ...

  applyAlgorithms();
  accept();
}

void PropertiesFrame::applyAlgorithms()
{
  GenericDataListBoxItem* cipherItem =
    dynamic_cast<GenericDataListBoxItem*>(encryptAlgo->listBox()->item(encryptAlgo->currentItem()));

  GenericDataListBoxItem* digestItem =
    dynamic_cast<GenericDataListBoxItem*>(signatureAlgo->listBox()->item(signatureAlgo->currentItem()));

  user->setInfo("CipherAlgo", (static_cast<std::string*>(cipherItem->getData()))->c_str());
  user->setInfo("DigestAlgo", (static_cast<std::string*>(digestItem->getData()))->c_str());
}

void PropertiesFrame::applyLanguages()
{
  user->setInfo("Language", langList->text(0).latin1());
  //   CryptonitGui& app = wxGetApp();
  //   app.setCurrentLocale(langList->GetStringSelection());
  //   user->setInfo("Language", langList->GetStringSelection().c_str());
}


void PropertiesFrame::buildCATree()
{
  if( user->authorities != NULL )
  {
    std::vector<std::string>* ca = user->authorities->getAllAuhtorityName();
    std::vector<std::string>::iterator itr;

    // Add algorithms for the isIssued() method
    OpenSSL_add_all_algorithms();

    hash_map<const char*, std::vector<std::string>, hash<const char*>, eqstr> adjacencies;
    std::vector<std::string> roots;

    for( itr = ca->begin(); itr != ca->end(); itr++ )
    {
      std::vector<std::string>::iterator i;

      // Load the father's certificate
      QString certFilename( (user->authorities->getAuthorityInfo( *itr, "certificate" )).c_str() );
      certFilename.prepend(QDir::separator());
      certFilename.prepend( user->getCACertificatesDir());

      Cryptonit::Certificate father;
      if( father.load( certFilename.latin1() ) != Cryptonit::SUCCESS )
      {
        return;
      }
      // If the certificate is self-signed,
      // add it in the roots list
      if( father.isIssued(father) == X509_V_OK )
      {
        roots.push_back( *itr );
      }

      // Force the 'itr' insertion into the 'adjacencies' list
      if( adjacencies[(*itr).c_str()].empty() );
      // If there is only one CA
      if( ca->size() == 1  && roots.size() == 0)
      {
        // Add it in the adjancies list, it will be inserted into tree
        // in the "orphans loop"

        // Force the 'itr' insertion into the 'adjacencies' list
        //if( adjacencies[(*itr).c_str()].empty() );
      }
      else
      {
        for( i = ca->begin(); i != ca->end(); i++ )
        {
          if( *i != *itr )
          {
            // Load the child certificate
            QString certFilename( (user->authorities->getAuthorityInfo( *i, "certificate" )).c_str() );
	    certFilename.prepend(QDir::separator());
            certFilename.prepend( user->getCACertificatesDir().c_str() );

            Cryptonit::Certificate child;
            if( child.load( certFilename.latin1() ) != Cryptonit::SUCCESS )
            {
              return;
            }

            // Force the 'itr' insertion into the 'adjacencies' list
            if( adjacencies[(*itr).c_str()].empty() );

            // Is 'itr' father of 'i'
            if( child.isIssued(father) == X509_V_OK )
            {
              // Add 'child' into the father's adjacencies list
              adjacencies[(*itr).c_str()].push_back(*i);
            }
          }
        }
      }
    }

    delete ca;

    // Construct the wxTreeCtrl
    caTree->clear();
    //     QListVireItem* rootItem = new QListViewItem(caTree);

    // Step into all 'roots'
    std::vector<std::string>::iterator root;
    for( root = roots.begin(); root != roots.end(); root++ )
    {

      // Load the certificate
      QString certFilename( (user->authorities->getAuthorityInfo( *root, "certificate" )).c_str() );
      certFilename.prepend( QDir::separator());
      certFilename.prepend( user->getCACertificatesDir().c_str() );

      Cryptonit::Certificate cert;
      QListViewItem* item;
      if( cert.load( certFilename.latin1() ) == Cryptonit::SUCCESS )
      {
        item = new GenericDataListViewItem(caTree,
                                           new CATreeItemData( certFilename.latin1(),
                                                               cert.getSubjectName().
                                                               getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',').c_str(),
                                                               cert.getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE, ' ').c_str() ),
                                           getCertName(cert).c_str());
      }
      else
      {
        item = new GenericDataListViewItem(caTree, 0, "Certificado no foi encontrado.");
      }
      addAllChilds( item, *root, adjacencies );
    }

    // Add all 'orphans' nodes
    hash_map<const char*,
    std::vector<std::string>,
    hash<const char*>,
    eqstr>::iterator orphan;

    GenericDataListViewItem* item;
    for( orphan = adjacencies.begin(); orphan != adjacencies.end(); orphan++ )
    {

      // Load the certificate
      QString certFilename( (user->authorities->getAuthorityInfo(orphan->first, "certificate")).c_str() );
      certFilename.prepend( QDir::separator() );
      certFilename.prepend( user->getCACertificatesDir().c_str() );

      Cryptonit::Certificate cert;
      if( cert.load( certFilename.latin1() ) == Cryptonit::SUCCESS )
      {
        //Exclui eventuais CAs raizes que eventualmente permaneceram na lista de rfos
        if( cert.isIssued(cert) != X509_V_OK ) {
		item = new GenericDataListViewItem(caTree,
						new CATreeItemData( certFilename.latin1(),
								cert.getSubjectName().
								getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',').c_str(),
								cert.getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE, ' ').c_str() ),
						getCertName(cert).c_str());
	}
      }
      else
      {
        item = new GenericDataListViewItem(caTree, 0, "Certificado no foi encontrado.");
      }
    }
  }
}


void PropertiesFrame::slotCA_onSelectCA(QListViewItem* litem)
{
  CATreeItemData* item = 
      static_cast<CATreeItemData*>(
      dynamic_cast<GenericDataListViewItem*>(litem)->getData());

  if ( item != NULL ) {
    Cryptonit::Certificate cert;
    if( cert.load( item->GetCertFilename().c_str(), Cryptonit::der_format ) == Cryptonit::SUCCESS ) {
      caDNValue->setText( item->GetShortDN().c_str() );
      caSNValue->setText( cert.getSubjectName().getValue("CN").c_str() );
      caINValue->setText( cert.getIssuerName().getValue("CN").c_str() );
    }
    btRemoveAuthority->setEnabled(true);
  }
}

void PropertiesFrame::slotCA_activateCA( QListViewItem * item )
{
    GenericDataListViewItem* _item = (GenericDataListViewItem*) item;
    if ( _item != NULL ) {
      CATreeItemData *itemData = (CATreeItemData *) _item->getData();
      Certificate cert;
   
      if( cert.load( itemData->GetCertFilename().c_str(), der_format ) == SUCCESS ) {
        QString title = "Propriedades do Certificado: ";
        title += itemData->GetShortDN().c_str();
   
        CertificateViewer cv( this );
	cv.setCaption(title);
	cv.init(cert, user, true);
        cv.exec();
        //cv.showModal(wxSize(435,500) , _("Close"), false);
      }
      btRemoveAuthority->setEnabled(TRUE);
  }
}

void PropertiesFrame::slotCA_viewCertDetails()
{

  GenericDataListViewItem* item = (GenericDataListViewItem*) caTree->selectedItem();
  if( item != NULL ) {
    CATreeItemData* itemData = (CATreeItemData*)item->getData();
    Certificate cert;

    if( cert.load( itemData->GetCertFilename().c_str(), der_format ) == SUCCESS ) {
      QString title = "Propriedades do Certificado: ";
      title += itemData->GetShortDN().c_str();

      CertificateViewer cv( this );
      cv.setCaption(title);
      cv.init(cert, user, true);
      cv.exec();

      //cv.showModal(wxSize(435,500) , _T("Close"), false);
    }
  }
}

void PropertiesFrame::slotCA_addCA()
{
  QString wildcard;
  wildcard =  "Todos os formatos suportados (*.cer; *.crt; *.der;*.pem);;";
  wildcard += "Certificado (*.cer; *.crt);;";
  wildcard += "Certificado codificado em DER (*.der);;";
  wildcard += "Certificado codificado em PEM (*.pem);;";

  QString filename = QFileDialog::getOpenFileName(
                       QDir::homeDirPath(),
                       wildcard,
                       this,
                       "open file dialog",
                       "Escolha uma autoridade certificadora para importar" );

  if( !filename.isEmpty() )
  {
    Cryptonit::Certificate certificate;
    if( certificate.load( filename ) == Cryptonit::SUCCESS )
    {
      if( ! certificate.isCA() )
      {
        int ret = QMessageBox::question(this, "Aviso", "O arquivo selecionado no parece ser um"
            "certificado de autoridade certificadora. Deseja continuar?", QMessageBox::Yes , QMessageBox::No);

        if( ret != QMessageBox::Yes )
        {
          return;
        }
      }
      std::string certHash = certificate.getHash();

      QString CAFilename( certHash.c_str() );
      CAFilename += ".der";
      CAFilename.prepend("/");
      CAFilename.prepend(user->getCACertificatesDir().c_str());

      if(certificate.save(CAFilename.latin1())!=0)
      {
        QString buffer = "No foi possvel gravar certificado no disco: ";
        buffer += CAFilename;
        QMessageBox::critical(this, "Erro", buffer);
        return;
      }
      else
      {
      
        std::string uri = certificate.extractCrlUri();
	
	std::cout << "uri: " << uri << std::endl;
	
          if( uri != "" && Crl::outDated(uri , user , &certificate)){
            int answer = QMessageBox::question(this, "Assinador",  "Voc deseja efetuar o download da CRL deste certificado?", QMessageBox::Yes,  QMessageBox::No);
            if( answer == QMessageBox::Yes ){ 
          
               QString newCrlFile = AddCrlDlg::fetchCrl( uri.c_str() );
	   
     	       std::cout << "newCrlFile: " << newCrlFile << std::endl;
               std::cout << "Exists: " << QFile::exists(newCrlFile) << std::endl;
               Crl newCrl;
	     
    	       if( newCrl.load( newCrlFile.latin1()) == SUCCESS ){
	   	  std::cout << "crl loaded: "<< std::endl;
		  AddCrlDlg::saveCrl( uri.c_str() , &newCrl , certificate , user ); 
		  std::cout << "crl saved: "<< std::endl;
        
    	       } else {
        	  QMessageBox::critical(this, "Assinador",  "A CRL baixada  invlida.");
    	       }
	       
	       if (!newCrlFile.isEmpty()) {
	          //QFile removeme(newCrlFile);
	          //removeme.remove();
    	       }
	     }
          } 
        QFileInfo fileInfo(CAFilename);
        user->authorities->addAuthority
        (certificate.getSubjectName().getValues
         (DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','),
         fileInfo.fileName().latin1());
        buildCATree();
      }
    }
    else
    {
      QString buffer = "No foi possvel carregar certificado: ";
      buffer += filename;
      QMessageBox::critical(this, "Erro", buffer);
      return;
    }
  }
}

void PropertiesFrame::slotCA_removeCA()
{
  if(!caTree->selectedItem()) return;
  
  CATreeItemData* item = 
      static_cast<CATreeItemData*>(
      dynamic_cast<GenericDataListViewItem*>(caTree->selectedItem())->getData());
  
  if( item != NULL )
  {
    QString msg = "Tem certeza que deseja remover a autoridade: ";
    msg +="\n";
    msg += item->GetShortDN().c_str();
    msg += " ?";

    int ret = QMessageBox::question(this, "Aviso", msg, QMessageBox::Yes, QMessageBox::No);

    if( ret != QMessageBox::Yes  )
    {
      return;
    }

    if( ! user->authorities->removeAuthority( item->GetDN() ) )
    {
      QString msg = "No foi possvel remover a autoridade: ";
      msg += item->GetDN().c_str();
      QMessageBox::critical(this, "Erro", msg);
      return;
    }

    QFile( item->GetCertFilename().c_str() ).remove();
    buildCATree();
    btRemoveAuthority->setEnabled(false);
  }
}

void PropertiesFrame::addAllChilds( QListViewItem *parent, std::string certDN, hash_map<const char *, std::vector<std::string>, hash<const char *>, eqstr> & adjacencies )
{
    if( adjacencies[certDN.c_str()].size() > 0 ) {
	std::vector<std::string>::iterator itr;

	for( itr = adjacencies[certDN.c_str()].begin();
	     itr != adjacencies[certDN.c_str()].end();
	     itr ++ ) {

	    // Load the certificate
	    QString certFilename( (user->authorities->getAuthorityInfo(*itr, "certificate")).c_str() );
	    certFilename.prepend( QDir::separator() );
	    certFilename.prepend( user->getCACertificatesDir().c_str() );

	    Cryptonit::Certificate cert;
	    QListViewItem *item;

	    if( cert.load( certFilename.latin1() ) == Cryptonit::SUCCESS ) {
		item = new GenericDataListViewItem(parent,
							   new CATreeItemData(certFilename.latin1(),
							      cert.getSubjectName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE , ',').c_str(),
							      cert.getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ' ').c_str() ),
                                           getCertName(cert).c_str());
	    } else {
	        item = new GenericDataListViewItem(parent, 0, "Certificado no foi encontrado.");
	    }
	    // Recursive call for all 'itr''s childs
	    addAllChilds( item, *itr, adjacencies );
	}
    }
    // Delete the current node from the 'adjacencies' list
    adjacencies[certDN.c_str()].clear();
    adjacencies.erase(certDN.c_str());
}

