/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include <qpixmap.h>  
#include <qdir.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qlistview.h>

#include "../Crl.hh"
#include "AddCrlDlg.h"


using namespace Cryptonit;

void VerifyCertDlg::setUser( Cryptonit::User * u )
{
  user = u;
}


void VerifyCertDlg::verify( std::vector<QString> certificatesFiles )
{
  std::vector<QString>::iterator certificatesFilesIt;
  std::vector<Certificate> *CAlist = new std::vector<Certificate>;

  //vector of CAs with out dated CRL;
  std::vector<Certificate> outdatedCRLCA;

  //loading CA
  QDir CADir(user->getCACertificatesDir().c_str());
  CADir.setFilter( QDir::Files);
  const QFileInfoList *list = CADir.entryInfoList();
  QFileInfoListIterator it( *list );
  QFileInfo *fi;
  while ( (fi = it.current()) != 0 )
  {
    Certificate cert;
    QString caCert = user->getCACertificatesDir().c_str();
    caCert += QDir::separator()+fi->fileName();
    if(cert.load(caCert.latin1())==SUCCESS)
    {
      CAlist->push_back(cert);
    }
    ++it;
  }

  // iterate through the reciptient's certificates ...
  for(certificatesFilesIt = certificatesFiles.begin(); certificatesFilesIt != certificatesFiles.end() ; certificatesFilesIt++)
  {
    Certificate c;
    if(c.load(certificatesFilesIt->latin1())==SUCCESS)
    {
      std::vector<Certificate> *CA= c.getPathToRoot(CAlist);

      //get a vector of crl
      //std::vector<Certificate>::iterator CAiter;
      std::vector<Crl> *CrlVector = new std::vector<Crl>;

      std::vector<Certificate>::reverse_iterator CArit = CA->rbegin();
      std::vector<Certificate>::iterator CAit = CA->begin();


      //will contain path from a certificate to the root CA
      std::vector<Certificate> *CaPath = new std::vector<Certificate>;

      std::vector<Crl>::iterator CrlIterator;
      std::vector<int> errorVector;
      int chainStatus = VERIF_CERTIFICATE_IS_VALID;
      int error = VERIF_CERTIFICATE_IS_VALID;

      // checking root CA
      if (CA->size() > 0) {
 Certificate cert( *CAit );
 CaPath->push_back( cert );
 
 //check if "root" CA is self signed
 if( CAit->isIssued(*CAit)!= X509_V_OK)
 {
  chainStatus = VERIF_ROOT_NOT_SELF_SIGNED ;
  errorVector.push_back( chainStatus );
  chainStatus = VERIF_CHAIN_NOT_VALID;
  std::cout << chainStatus << std::endl;
 }
 else
 {
  errorVector.push_back( chainStatus );
 }
      }
      else {
       chainStatus = VERIF_CERTIFICATE_NO_SIGNER_FOUND;
 errorVector.push_back( chainStatus );
      }



      //checking other CAs (and user certificate) in the chain
      if (CA->size() > 0) {
       CAit++;
      }
      for( ; CAit != CA->end() ;CAit++)
      {
        Certificate cert( *CAit );
        //CaPath->push_back( cert );
        if( user->authorities->getAuthority(CAit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',')) )
        {
          std::string crlBase = user->authorities->getAuthority(
                                  CAit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','))->getAttributeFirstValue( "CRL" );
          Crl *c = new Crl();;
          if( crlBase != "" )
          {
            QString crlFilename = QString(user->getCRLDir().c_str()) + QDir::separator() +QString(crlBase.c_str());
            
            c->load(crlFilename.latin1());
          }
          CrlVector->push_back(*c);
        }


        if( !(( error = CAit->isValid(CrlVector, CaPath)) & VERIF_CERTIFICATE_IS_VALID ) )
        {	
          if( error & VERIF_CRL_NOT_FOUND || error & VERIF_CRL_IS_OUT_DATED)
          {//CRL_NOT_FOUND and VERIF_CRL_IS_OUT_DATED are just warnings not errors
            chainStatus |= error ;
            if( error & VERIF_CRL_IS_OUT_DATED )
            {
              outdatedCRLCA.push_back( cert );
            }
          }
          else
          {
	  
            chainStatus |= VERIF_CHAIN_NOT_VALID | error ;
          }
          errorVector.push_back( chainStatus );
        }
        else
        {	
          if( error & VERIF_CRL_IS_OUT_DATED )
          {
            outdatedCRLCA.push_back( cert );
          }
          if( chainStatus & VERIF_CHAIN_NOT_VALID )
            error |= VERIF_CHAIN_NOT_VALID;

          errorVector.push_back( error );
        }
        CaPath->push_back( cert );
      }

 
    if( outdatedCRLCA.size() ){
       int response = QMessageBox::question(
            this,
            "Ateno",
     "Algumas CRLs expiraram.\nDeseja atualiz-las e reiniciar o processo de verificao?",
            "Sim", "No",QString::null, QMessageBox::Yes, QMessageBox::No );

 if(response == 0)
 {
 
    std::vector<Certificate>::iterator crlit;
    int ret = Cryptonit::SUCCESS;
    
    for( crlit = outdatedCRLCA.begin() ; crlit != outdatedCRLCA.end() ; crlit++ )
    { 
  
       std::string crlURI = user->authorities->getAuthority(crlit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','))->getAttributeFirstValue( "URI" );   
     
       ret = AddCrlDlg::updateCrl(crlURI , *crlit , user );      
       if (ret) {
		std::string crlBase = user->authorities->getAuthority(
			crlit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','))->getAttributeFirstValue( "CRL" );
		AddCrlDlg::deleteCrl(crlBase, *crlit , user );
       }
    }
    outdatedCRLCA.clear();
    CAit = CA->begin();
    //checking other CAs (and user certificate) in the chain
    //checking other CAs (and user certificate) in the chain
      if (CA->size() > 0) {
       CAit++;
      }
    for( ; CAit != CA->end() ;CAit++){
      Certificate cert( *CAit );
      //CaPath->push_back( cert );
      
  if( user->authorities->getAuthority(CAit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',')) ){
      std::string crlBase = user->authorities->getAuthority(
           CAit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','))->getAttributeFirstValue( "CRL" );
      Crl *c = new Crl();
      if( crlBase != "" ){
       QString crlFilename = QString(user->getCRLDir().c_str())+QDir::separator()+QString(crlBase.c_str());
        c->load(crlFilename);
      }
      CrlVector->push_back(*c);
  }  
  
  
  if( !(( error = CAit->isValid(CrlVector, CaPath)) & VERIF_CERTIFICATE_IS_VALID ) ){  
      if( error & VERIF_CRL_NOT_FOUND || error & VERIF_CRL_IS_OUT_DATED) {
        //CRL_NOT_FOUND and VERIF_CRL_IS_OUT_DATED are just warnings not errors
        chainStatus |= error ;
        if( error & VERIF_CRL_IS_OUT_DATED ){
           outdatedCRLCA.push_back( cert ); 
        }	
      } else {        
	chainStatus |= VERIF_CHAIN_NOT_VALID | error ;
      }      
      errorVector.push_back( chainStatus );
  } else{    
      if( error & VERIF_CRL_IS_OUT_DATED ){      
   outdatedCRLCA.push_back( cert );
      }
   if( chainStatus & VERIF_CHAIN_NOT_VALID )
       error |= VERIF_CHAIN_NOT_VALID;

      errorVector.push_back( error );
  }
  
  CaPath->push_back( cert );
 }
    
   
 }
    }

      // checking user certificate
      bool ok = true;
      for(CrlIterator = CrlVector->begin() ; CrlIterator != CrlVector->end() ; CrlIterator++)
      {
        Crl* tempCrl = new Crl( *CrlIterator );
        ok |= !CArit->isRevoked( tempCrl );
        delete tempCrl;
      }



      QListViewItem *caId = NULL;
      std::vector<QListViewItem*> treeItems; // use for expanding tree's items (damn wxWindows ... )

      //adding messages in the tree
      QString certificateName ;
      certificateName = user->addressBook->getContactInfo(CArit->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ','), "cn").c_str();


      //trying ti get a human readable certificate's name.
      if(certificateName == "")
        certificateName = CArit->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ',').c_str()
                          ;
      certificateName = "Certificado de " + certificateName;

      std::vector<int>::reverse_iterator errorIt = errorVector.rbegin();
      unsigned int index = 0;
      //Cris as colunas dinamicas da Tree
      for (;index < CA->size(); index++) {
        QString nivel = QString("Nivel %1").arg(index) ;
       tree->addColumn(nivel);
 tree->header()->setLabel( index, nivel);
      }
      tree->setSortColumn(-1);
      for( ; CArit != CA->rend() ;CArit++)
      {
        //Simular o efeito de wxTR_TWIST_BUTTONS existente no wxWindows
 if (caId != NULL && (caId->parent() !=NULL)) {
  caId = caId->parent();
 }
        // checking user certificate
        bool ok = true;
        for(CrlIterator = CrlVector->begin() ; CrlIterator != CrlVector->end() ; CrlIterator++)
        {
          Crl* tempCrl = new Crl( *CrlIterator );
          ok |= !CArit->isRevoked( tempCrl );
          delete tempCrl;
        }



        certificateName = user->addressBook->getContactInfo(CArit->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ','), "cn").c_str();
        //trying to get a human readable certificate's name.
        if(certificateName == "")
          certificateName = CArit->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ',').c_str();
        certificateName = "Certificado de " + certificateName;

        if (!ok)
        {
          caId = addOnRoot(certificateName, 3);
          treeItems.push_back( caId );
          addItem(caId, "Este certificado foi revogado", 0);

        }
        else if(*errorIt & VERIF_CHAIN_NOT_VALID)
        {
          caId = addItem(caId, certificateName,3);
          treeItems.push_back( caId );

          QListViewItem *tmpId = addItem(caId, "A cadeia de certificados no  vlida", 0);
          treeItems.push_back( tmpId );
          if(*errorIt & VERIF_CRL_NO_SIGNER_FOUND)
          {
            addItem(tmpId, "Este certificado  invlido. Nenhum assinate foi encontrado para a CRL desta CA.", 0);
          }
          if(*errorIt & VERIF_CERTIFICATE_IS_REVOKED)
          {
            addItem(tmpId, "Este certificado  invlido. Certificado revogado.", 0);
          }
        }
        else if((*errorIt & VERIF_CERTIFICATE_IS_VALID) && !(*errorIt & VERIF_ROOT_NOT_SELF_SIGNED) )
        {

          caId = addItem(caId, certificateName, 3);
          treeItems.push_back( caId );
          addItem(caId, "Este certificado  vlido", 1);
        }

        else if(*errorIt & VERIF_ROOT_NOT_SELF_SIGNED)
        {
          caId = addItem(caId, certificateName, 3);
          treeItems.push_back( caId );
          QListViewItem *tmpId = addItem(caId, "Este certificado  invlido", 0);
          treeItems.push_back( tmpId );
          addItem(tmpId, "Este certificado raiz no  auto-assinado", 0);
        }
        else if(*errorIt & VERIF_CERTIFICATE_NO_SIGNER_FOUND)
        {
          caId = addItem(caId, certificateName, 3);
          treeItems.push_back( caId );
          QListViewItem *tmpId = addItem(caId, "Este certificado  invlido", 0);
          treeItems.push_back( tmpId );
          addItem(tmpId, "O emissor deste certificado  desconhecido", 0);
        }
        if( *errorIt & VERIF_CRL_IS_OUT_DATED )
        {
          addItem(caId, "CRL expirada.", 2);

        }
        else if( *errorIt & VERIF_CRL_NOT_FOUND )
        {
	  addItem(caId, "CRL no encontrada",2);
        }
        errorIt++;
      }

      std::vector<QListViewItem*>::iterator it;
      for( it = treeItems.begin() ; it != treeItems.end() ; it++ )
      {
       QListViewItem *item = (QListViewItem*)(*it);
 item->setOpen(TRUE);
      }
      CA->clear();
      delete CA;
    }
  }

  outdatedCRLCA.clear();
}


QListViewItem* VerifyCertDlg::addItem( QListViewItem *parent, QString value, int imgIdx)
{
    QListViewItem *item = NULL;
    if (parent != NULL) {
 QPixmap *pixmap = selectImage(imgIdx);
     item = new QListViewItem(parent, value);
     if (pixmap != NULL) {
      item->setPixmap(0, *pixmap);
 }    
    }
    else {
     item = addOnRoot(value, imgIdx);
    }
    return item;
}


QListViewItem* VerifyCertDlg::addOnRoot( QString value, int imgIdx )
{
    QPixmap *pixmap = selectImage(imgIdx);
    QListViewItem *item = new QListViewItem(tree, value);
    if (pixmap != NULL) {
     item->setPixmap(0, *pixmap);
    }    
    return  item;
}

QPixmap* VerifyCertDlg::selectImage(int imgIdx) 
{
    QPixmap *pixmap = NULL;
    switch(imgIdx) {
     case 0: {
  pixmap = &image0; //Failure
  break;
 }
     case 1: {
  pixmap = &image1; //Success
  break;
 }
     case 2: {
  pixmap = &image2; //Warning
  break;
 }
     case 3: {
  pixmap = &image3; //certificate
 }
 }  
    return pixmap;
}


void VerifyCertDlg::init()
{
    certLabel->hide();
    tree->header()->hide();
}

void VerifyCertDlg::verifySignature( int validSignature, QString filename, std::vector<Cryptonit::Certificate> * CA )
{
    
    //get a vector of crl
    std::vector<Certificate>::iterator CAit;
    std::vector<Crl> *CrlVector = new std::vector<Crl>;
    
    //vector of CAs with out dated CRL;
    std::vector<Certificate> outdatedCRLCA;

    std::vector<Certificate>::reverse_iterator CArit = CA->rbegin();
    CAit = CA->begin();

    //will contain path from a certificate to the root CA
    std::vector<Certificate> *CaPath = new std::vector<Certificate>;

    std::vector<Crl>::iterator CrlIterator;

    // checking root CA 
    Certificate cert( *CAit );
    CaPath->push_back( cert );

    
    std::vector<int> errorVector;
    int chainStatus = VERIF_CERTIFICATE_IS_VALID;
    int error = VERIF_CERTIFICATE_IS_VALID;
    
    //check if "root" CA is self signed 
    if( cert.isIssued( cert )!= X509_V_OK){
       chainStatus= VERIF_ROOT_NOT_SELF_SIGNED ;
       errorVector.push_back( chainStatus );
       chainStatus = VERIF_CHAIN_NOT_VALID;
    } else {
       errorVector.push_back( chainStatus );
    }
    
  //checking other CAs (and user certificate) in the chain
  //checking other CAs (and user certificate) in the chain
      if (CA->size() > 0) {
       CAit++;
      }
    for( ; CAit != CA->end() ;CAit++){
      Certificate cert( *CAit );
      //CaPath->push_back( cert );
      
  if( user->authorities->getAuthority(CAit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',')) ){
      std::string crlBase = user->authorities->getAuthority(
           CAit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','))->getAttributeFirstValue( "CRL" );
      Crl *c = new Crl();
      if( crlBase != "" ){
       QString crlFilename = QString(user->getCRLDir().c_str())+QDir::separator()+QString(crlBase.c_str());
        c->load(crlFilename);
      }
      CrlVector->push_back(*c);
  }  
  
  
  if( !(( error = CAit->isValid(CrlVector, CaPath)) & VERIF_CERTIFICATE_IS_VALID ) ){  
      if( error & VERIF_CRL_NOT_FOUND || error & VERIF_CRL_IS_OUT_DATED) {
        //CRL_NOT_FOUND and VERIF_CRL_IS_OUT_DATED are just warnings not errors
        chainStatus |= error ;
        if( error & VERIF_CRL_IS_OUT_DATED ){
           outdatedCRLCA.push_back( cert ); 
        }	
      } else {        
	chainStatus |= VERIF_CHAIN_NOT_VALID | error ;
      }      
      errorVector.push_back( chainStatus );
  } else{    
      if( error & VERIF_CRL_IS_OUT_DATED ){      
   outdatedCRLCA.push_back( cert );
      }
   if( chainStatus & VERIF_CHAIN_NOT_VALID )
       error |= VERIF_CHAIN_NOT_VALID;

      errorVector.push_back( error );
  }
  
  CaPath->push_back( cert );
 } 
 
 
    if( outdatedCRLCA.size() ){
       int response = QMessageBox::question(
            this,
            "Ateno",
     "Algumas CRLs expiraram.\nDeseja atualiz-las e reiniciar o processo de verificao?",
            "Sim", "No",QString::null, QMessageBox::Yes, QMessageBox::No );

 if(response == 0)
 {
 
    std::vector<Certificate>::iterator crlit;
    int ret = Cryptonit::SUCCESS;
    
    for( crlit = outdatedCRLCA.begin() ; crlit != outdatedCRLCA.end() ; crlit++ )
    { 
  
       std::string crlURI = user->authorities->getAuthority(crlit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','))->getAttributeFirstValue( "URI" );   
     
       ret = AddCrlDlg::updateCrl(crlURI , *crlit , user );      
       if (ret) {
		std::string crlBase = user->authorities->getAuthority(
			CAit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','))->getAttributeFirstValue( "CRL" );
		AddCrlDlg::deleteCrl(crlBase, *crlit , user );
       }
    }
    outdatedCRLCA.clear();
    CAit = CA->begin();
    //checking other CAs (and user certificate) in the chain
    //checking other CAs (and user certificate) in the chain
      if (CA->size() > 0) {
       CAit++;
      }
    for( ; CAit != CA->end() ;CAit++){
      Certificate cert( *CAit );
      //CaPath->push_back( cert );
      
  if( user->authorities->getAuthority(CAit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ',')) ){
      std::string crlBase = user->authorities->getAuthority(
           CAit->getIssuerName().getValues(DN_DISPLAY_LONG | DN_DISPLAY_VALUE, ','))->getAttributeFirstValue( "CRL" );
      Crl *c = new Crl();
      if( crlBase != "" ){
       QString crlFilename = QString(user->getCRLDir().c_str())+QDir::separator()+QString(crlBase.c_str());
        c->load(crlFilename);
      }
      CrlVector->push_back(*c);
  }  
  
  
  if( !(( error = CAit->isValid(CrlVector, CaPath)) & VERIF_CERTIFICATE_IS_VALID ) ){  
      if( error & VERIF_CRL_NOT_FOUND || error & VERIF_CRL_IS_OUT_DATED) {
        //CRL_NOT_FOUND and VERIF_CRL_IS_OUT_DATED are just warnings not errors
        chainStatus |= error ;
        if( error & VERIF_CRL_IS_OUT_DATED ){
           outdatedCRLCA.push_back( cert ); 
        }	
      } else {        
	chainStatus |= VERIF_CHAIN_NOT_VALID | error ;
      }      
      errorVector.push_back( chainStatus );
  } else{    
      if( error & VERIF_CRL_IS_OUT_DATED ){      
   outdatedCRLCA.push_back( cert );
      }
   if( chainStatus & VERIF_CHAIN_NOT_VALID )
       error |= VERIF_CHAIN_NOT_VALID;

      errorVector.push_back( error );
  }
  
  CaPath->push_back( cert );
 }
    
   
 }
    }
     
     
       
    bool signerCertOk = true;
    for(CrlIterator = CrlVector->begin() ; CrlIterator != CrlVector->end() ; CrlIterator++){    
       Crl *tempCrl = new Crl( *CrlIterator );
       signerCertOk |= !CArit->isRevoked( tempCrl );
       delete tempCrl;
    }
    

    QListViewItem *caId, *rootId ;

    // store each QListViewItem in a vector in order to expand the tree at the end.
    std::vector<QListViewItem*> treeItems; 

    bool badsigner = false;

    QString certificateName ;
    certificateName = user->addressBook->getContactInfo(CArit->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ','), "cn").c_str();
 
    if(certificateName == "")
       certificateName = CArit->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ',').c_str();
      
    if((validSignature == SUCCESS) && signerCertOk && !(chainStatus & VERIF_CHAIN_NOT_VALID)){
 	caId = addOnRoot("Assinatura de " + filename + "  vlida" ,1);
        treeItems.push_back( caId );
        rootId = caId;

    } else {
       caId = addOnRoot("Assinatura de " + filename + "  invlida" ,0);
       rootId = caId;
       treeItems.push_back( caId );
       if(validSignature != SUCCESS){
            if(validSignature == PKCS7_BAD_SIGNER){
               addItem(caId, "Este arquivo deve ter sido assinado por "+ certificateName ,0);
               badsigner = true;
            } else if (validSignature == PKCS7_SIGNATURE_VERIF_FAILED){
               addItem(caId, "Verificao da assinatura falhou" ,0);
            } 
       }/*else if(!signerCertOk){
             addItem(caId,_("The signer certificate is revoked ")+ certificateName ,0);
     
 		}else if(chainStatus & VERIF_CHAIN_NOT_VALID){
     		addItem(caId,_("The certificate chain is not valid") ,0);
 		}else if(chainStatus & VERIF_CRL_NOT_FOUND ){
     		addItem(caId,_("The certificate chain is not valid") ,0);
 		}*/
    }
 
    
    if(!badsigner){
 //adding messages in the tree
 if(signerCertOk && !(chainStatus & VERIF_CHAIN_NOT_VALID)){
     caId = addItem(caId, "Assinado por " + certificateName , 3);
     if( chainStatus & VERIF_CRL_NOT_FOUND )
  addItem(caId, "Esta deve ser vlida porm alguns alertas foram gerados." ,2);
   /*  else 
  addItem(caId, _("This certificate is valid"),1);*/
     treeItems.push_back( caId );
    
 } else {
     caId = addItem(caId, "Assinado por " + certificateName , 3);
     treeItems.push_back( caId );
 }
     
    
 int index = 0;
 for (;index < CA->size(); index++) {
  QString nivel = QString("Nivel %1").arg(index) ;
  tree->addColumn(nivel);
  tree->header()->setLabel( index, nivel);
 }
 tree->setSortColumn(-1);

 std::vector<int>::reverse_iterator errorIt = errorVector.rbegin(); 
 for( ; CArit != CA->rend() ;CArit++){     
     bool signerCertOk = true;
     for(CrlIterator = CrlVector->begin() ; CrlIterator != CrlVector->end() ; CrlIterator++){
       Crl *tempCrl = new Crl( *CrlIterator );
       signerCertOk |= !CArit->isRevoked( tempCrl );
       delete tempCrl;
     }
     QString certificateName ;
     certificateName = user->addressBook->getContactInfo(CArit->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ','), "cn").c_str();
     
     if(certificateName == "")
  certificateName = CArit->getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ',').c_str();

     certificateName = "Certificado de " + certificateName;
     caId = addItem(caId, certificateName,3);
     treeItems.push_back( caId );

     if (!signerCertOk){
  caId = addOnRoot(certificateName,3);
  treeItems.push_back( caId );
  addItem(caId, "Este certificado foi revogado", 0);
     } else if(*errorIt & VERIF_CHAIN_NOT_VALID){
  addItem(caId, "A cadeia de certificao  invlida", 0);
  if(*errorIt & VERIF_CRL_NO_SIGNER_FOUND){
      addItem(caId, "Este certificado  invlido. Nenhum emissor foi encontrado para a CRL desta CA.", 0);
  }
     } else if(*errorIt & VERIF_CERTIFICATE_IS_VALID){
  addItem(caId, "Este certificado  vlido", 1);     
     } else if(*errorIt & VERIF_CRL_NO_SIGNER_FOUND){
  addItem(caId, "Este certificado  invlido. Nenhum emissor foi encontrado para a CRL desta CA.", 0);     
     } else if(*errorIt & VERIF_CERTIFICATE_IS_REVOKED){
  addItem(caId, "Este certificado  invlido. Certificado revogado.", 0);
     }
     if(*errorIt & VERIF_ROOT_NOT_SELF_SIGNED){
  addItem(caId, "Este certificado  auto-assinado.",0);
     }     
     if( *errorIt & VERIF_CRL_IS_OUT_DATED ){
  addItem(caId, "CRL expirada.", 2);
  
     }else  if( *errorIt & VERIF_CRL_NOT_FOUND ){     
  addItem(caId, "CRL no encontrada.",2);
     }
     errorIt++; 
 }
 
 //expand all items in the tree (wx sux)
 std::vector<QListViewItem*>::iterator it;
        for( it = treeItems.begin() ; it != treeItems.end() ; it++ )
        {
           QListViewItem *item = (QListViewItem*)(*it);
     item->setOpen(TRUE);
        }
    }

    
    outdatedCRLCA.clear();
    

}
