/*
 * The Cryptonit security software suite is developped by IDEALX
 * Cryptonit Team (http://IDEALX.org/ and http://cryptonit.org).
 *
 * Copyright ? 2003 IDEALX
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */



#ifndef _pkcs10_hh_
#define _pkcs10_hh_
 
#include <vector>
#include <string>

#include <openssl/x509.h>

#include "Certificate.hh"
#include "pkcs8.hh"


namespace Cryptonit 
{

    class pkcs10
    {

    private:
	X509_REQ* request;

	EVP_PKEY* privateKey;
	std::string currentPrivateKeyFilename;

	X509_NAME* subject;

    public:
	
	pkcs10();

	pkcs10( pkcs10& src );

	pkcs10( const char* pkeyFilename, const char* password );

	pkcs10( const char* pkeyFilename, const char* password,
			    const std::vector< std::pair< std::string, std::string > >& infos );

	~pkcs10(); 

	int setPrivateKey( const char* pkeyFilename, const char* password );

	int setInformations( const std::vector< std::pair< std::string, std::string > >& infos );

	int write( const char* filename );
	int write( FILE* fp );

	int load( const char* filename );

	bool matchCertificate( Certificate& certificate );

	bool matchPrivateKey( pkcs8& pkey );

    };
}
#endif
