//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "tokenkey.h"
#include <openssl/evp.h>
#include <openssl/safestack.h>
#include <openssl/engine.h>
#include <string>
#include <cstdio>
#include <iostream>

TokenKey::TokenKey(std::string keyID, std::string password)
    : Key()
{
    //Configurao da ENGINE SSL.
    STACK *pre_cmds = sk_new_null();
    sk_push(pre_cmds, "SO_PATH:/usr/lib/opensc/engine_pkcs11.so");
    sk_push(pre_cmds, "ID:pkcs11");
    sk_push(pre_cmds, "LIST_ADD:1");
    sk_push(pre_cmds, "LOAD");
    sk_push(pre_cmds, "MODULE_PATH:/usr/lib/pkcs11/opensc-pkcs11.so");
    char *pin = (char*)malloc(sizeof(char)*(4+password.size()));
    sprintf(pin, "PIN:%s", password.c_str());
    sk_push(pre_cmds, pin);
    e = load_engine (pre_cmds);
    //Obtm a chave
    std::string keyId = "slot_0-id_"+keyID;
    PW_CB_DATA cb_data;
    cb_data.password = password.c_str();
    cb_data.prompt_info = keyId.c_str();
    key = ENGINE_load_private_key(e, keyId.c_str(), NULL, &cb_data);
}


TokenKey::~TokenKey()
{
  free();
}

void TokenKey::free()
{
  if (key != NULL) {
  	EVP_PKEY_free(key);
  }
  ENGINE_finish(e);
  ENGINE_free(e);
  std::cout << __FUNCTION__ << " executed" << std::endl;
}

ENGINE *TokenKey::load_engine (STACK *pre_cmds)
{
  ENGINE *e;

  ENGINE_load_builtin_engines();
  e = ENGINE_by_id("dynamic");
  if (e == NULL)
  {
    std::cout << "[Error]: Cannot load an engine with the name dynamic\n";
    return NULL;
  }

  /* preconfigure engine */
  if (! configure_engine (e, pre_cmds))
  {
    std::cout << "[Error]: Cannot preconfigure engine\n";
    ENGINE_free(e);
    return NULL;
  }

  /* init engine */
  if(!ENGINE_init(e))
  {
    std::cout << "[Error]: Engine initialisation failed\n";
    ENGINE_free(e);
    return NULL;
  }

  return e;
}


int TokenKey::configure_engine (ENGINE *e, STACK *cmds)
{
  int loop, res, num = sk_num(cmds);
  if(num < 0)
  {
    std::cout << "[Error]: internal stack error\n";
    return 0;
  }
  for(loop = 0; loop < num; loop++)
  {
    char buf[256];
    const char *cmd, *arg;
    cmd = sk_value(cmds, loop);
    res = 1; /* assume success */
    /* Check if this command has no ":arg" */
    if((arg = strstr(cmd, ":")) == NULL)
    {
      if(!ENGINE_ctrl_cmd_string(e, cmd, NULL, 0))
        res = 0;
    }
    else
    {
      if((int)(arg - cmd) > 254)
      {
        std::cout << "[Error]: command name too long\n";
        return 0;
      }
      memcpy(buf, cmd, (int)(arg - cmd));
      buf[arg-cmd] = '\0';
      arg++; /* Move past the ":" */
      /* Call the command with the argument */
      if(!ENGINE_ctrl_cmd_string(e, buf, arg, 0))
        res = 0;
    }
    if(!res)
    {
      std::cout << "[Failure]: " << cmd << "\n";
      return 0;
    }
  }
  return 1;
       }

EVP_PKEY* TokenKey::getPrivateKey() {
	return key;
}
