#!/bin/bash

openssl_refmaj=0
openssl_refmin=9
openssl_reffine="7e"
update_openssl=""

openssl_version=`openssl version | cut -f2 -d\ `
openssl_maj=`echo $openssl_version | cut -f1 -d.`
openssl_min=`echo $openssl_version | cut -f2 -d.`
openssl_fine=`echo $openssl_version | cut -f3 -d.`
echo " found version $openssl_maj.$openssl_min.$openssl_fine. of OpenSSL"
if [ \( \( $openssl_maj -eq $openssl_refmaj \) -a \( $openssl_min -eq $openssl_refmin \) -a \( \( $openssl_fine == $openssl_reffine \) -o \( $openssl_fine \> $openssl_reffine \) \) \) -o \( \( $openssl_maj -eq $openssl_refmaj \) -a \( $openssl_min -gt $openssl_refmin \) \) -o \( $openssl_maj -gt $openssl_refmaj \) ];
then
	echo OK!
else
	echo Version too old!
	update_openssl="yes"
fi	     
if [ ! -z $update_openssl ];
then
	rpm -U --nodeps libopenssl0.9.7-0.9.7e-5mdk.i586.rpm
	rpm -U --nodeps libopenssl0.9.7-devel-0.9.7e-5mdk.i586.rpm
	rpm -U openssl-0.9.7e-5mdk.i586.rpm
fi

# Verificar se o PCSC-lite está instalado e se está atualizado.
pcsc_running=`pgrep pcscd | head -n 1`
if [ ${#pcsc_running} -gt 0 ];
then
	pcsc_exe=`ps -aux | grep pcscd | head -n 1 | cut -f28 -d\ `
else
	if [ -x /sbin/pcscd ]; then
		pcsc_exe=/sbin/pcscd
        elif [ -x /usr/sbin/pcscd ]; then
		pcsc_exe=/usr/sbin/pcscd
	elif [ -x /usr/local/sbin/pcscd ]; then
		pcsc_exe=/usr/local/sbin/pcscd
	fi
fi

echo "PCSCD path: ($pcsc_exe)" 
if [ ${#pcsc_exe} -gt 0 ];
then
	pcsc_version=`$pcsc_exe --version | head -n 1 | cut -f3 -d\ `
	pcsc_maj=`echo $pcsc_version | cut -f1 -d.`
	pcsc_min=`echo $pcsc_version | cut -f2 -d.`
	pcsc_fine=`echo $pcsc_version | cut -f3 -d.`
	echo " found version $pcsc_maj.$pcsc_min.$pcsc_fine. in $pcsc_exe"	
	if [ `expr index "$pcsc_fine" "-"` -gt 0 ];
	then
		echo "Testar beta informations"
		index=`expr index "$pcsc_fine" "-"`
		index=$(( $index + 4 ))
		pcsc_beta_number=${pcsc_fine:index}
		echo "Beta number: $pcsc_beta_number"
	fi
#	value=$(( $pcsc_maj * 1000 ))
 	value=$(( (( $pcsc_maj * 1000 )) + (( $pcsc_min * 100 )) + 90 ))
	if [ -n $pcsc_beta_number ];
	then 
		value=$(( $value + $pcsc_beta_number ))
	fi
	echo "value = $value"
	if [ $value -lt 1297 ]; #versão 1.2.9-beta7
	then
		rpm -U --nodeps pcsc-lite-1.2.9-beta7.i586.rpm
	fi
else 
	rpm -ivh pcsc-lite-1.2.9-beta7.i586.rpm
fi

# Verifica se o CCID está instalado e se precisa ser atualizado
echo "Instalao do driver CCID"
ccid_pkg=`rpm -qa | grep ^ccid | head -n 1`
if [ ${#ccid_pkg} -gt 0 ];
then
	rpm -Fvh ccid-0.9.3-1.i586.rpm
	echo "Atualizado (old $ccid_pkg)."
else
	rpm -ivh ccid-0.9.3-1.i586.rpm
	echo "Instalado."
fi

# Verifica se o OpenSC está instalado e se precisa ser atualizado

opensc_pkg=`rpm -qa | grep ^opensc | head -n 1`
echo "Instalkao do OpenSC"
if [ ${#opensc_pkg} -gt 0 ];
then
	rpm -Fvh --nodeps opensc-0.9.6-beta7.i586.rpm
	echo "Atualizado (old $opensc_pkg)."
else
	rpm -Uvh --nodeps libsasl2-devel-2.1.19-5mdk.i586.rpm
	rpm -Uvh --nodeps libldap2-devel-2.1.30-3mdk.i586.rpm
	rpm -ivh --nodeps opensc-0.9.6-beta7.i586.rpm
	echo "Instalado."
fi


# Instala o Assinador
echo "Instalao do Assinador"
assinador_pkg=`rpm -qa | grep ^assinador | head -n 1`
if [ ${#assinador_pkg} -gt 0 ];
then
	rpm -Fvh assinador-1.0.6-1.i586.rpm
	echo "Atualizado (old $assinador_pkg)."
else
	rpm -ivh assinador-1.0.6-1.i586.rpm
	echo "Instalado"
fi
